struct ArrayBasic
{
	boolean var_array_boolean[8];
	char var_array_char[8];
    octet var_array_octet[8];

	short var_array_short[8];
    unsigned short var_array_ushort[8];

	long var_array_long[8];
	unsigned long var_array_ulong[8];

	long long var_array_longlong[8];
	unsigned long long var_array_ulonglong[8];

	float var_array_float[8];
	double var_array_double[8];
};

