struct SequenceInner
{
    short var_short;
    long var_long;
};

struct SequenceCustom
{
    sequence<SequenceInner> var_sequence_inner;
    sequence<SequenceInner, 8> var_sequence_inner_8;
};

module a_module
{
    typedef sequence<SequenceInner> SequenceInnerSeq;

    struct ModuleSequenceCustom
    {
        sequence<SequenceInner> var_seq_seq;
        SequenceInnerSeq var_alias_seq;
    };
};