bitset MyBitset
{
	bitfield<3> a;
	bitfield<1> b;
	bitfield<4>;
	bitfield<10> c;
	bitfield<12, short> d;
};

@bit_bound(8)
bitmask MyBitMask
{
	@position(0) flag0,
	@position(1) flag1,
	@position(4) flag4,
	@position(6) flag6
};


struct BitsetBitmask
{
	MyBitset my_bitset;
	MyBitMask my_bitmask;
};

struct BM2 : BitsetBitmask
{
	MyBitMask two;
	long mylong;
};
