struct Basic
{
	boolean var_boolean;
	char var_char;
    octet var_octet;

	short var_short;
    unsigned short var_ushort;

	long var_long;
	unsigned long var_ulong;

	long long var_longlong;
	unsigned long long var_ulonglong;

	float var_float;
	double var_double;
};
